#! /usr/bin/env python
import os
import sys
import re
import zipfile
import shutil
import urllib2
import subprocess
import tempfile
import urllib


class BaseInstaller(object):
    wpm_meta_path = os.path.expanduser('~/.wpm/')
    plugin_db_path = os.path.join(wpm_meta_path, 'available_plugins')
    tmp_dir = tempfile.mkdtemp()
    plugins_svn = "http://plugins.svn.wordpress.org"
    target_location = None
    plugin_name = ''

    def __init__(self, *args, **kwargs):

        for key, value in kwargs.items():
            setattr(self, key, value)

    def download_file(self, url):
        """ Common method to download a file from a given url """

        print 'Fetching %s' % url

        temp = tempfile.mkstemp()

        urllib.urlretrieve(url, temp[1])

        return temp[1]

    def extract(self, file):
        """ Extract the contents of a zip file """

        print "Unzipping %s" % file

        z = zipfile.ZipFile(file)

        z.extractall(self.tmp_dir)

        return z.namelist()


    def set_security_keys(self, config_file=None):
        """ Generates new security keys and writes them
            to the wp-config.php file """

        if not config_file: return

        print "Generating new security keys"

        try:
            keys = urllib2.urlopen(
                'https://api.wordpress.org/secret-key/1.1/salt/').readlines()
        except:
            print "Couldn't generate security keys from WordPress API. You'll have to set them yourself."
            return

        config_file = open(self.target_location + '/wp-config.php', 'r+')

        lines = config_file.readlines()

        for i in range(len(lines)):
            for key in keys:
                match = re.findall("^define\('(.*)',", key)
                if match is not None and match[0] in lines[i]:
                    lines[i] = key

        config_file.seek(0)

        config_file.writelines(lines)

    def run_command(self, cmd_list):
        """ Runs a shell command command """

        proc = subprocess.Popen(cmd_list, stdout=subprocess.PIPE)

        for line in proc.stdout:
            if line is not "\r\n":
                print line

    def move_tmp(self, src, dest):
        """ Renames the tmp installation dir to the
            specified destination """

        if os.path.exists(os.path.abspath(dest)) and not self.overwrite:
            print "Installation already exists"
            return False

        try:
            shutil.rmtree(os.path.abspath(dest))
        except:
            pass

        shutil.move(src, dest)

    def install(self):

        if os.path.exists(os.path.join(self.target_location, self.plugin_name)) and not self.overwrite:
            sys.exit("Target location exists. Aborting.")



class ZIPInstaller(BaseInstaller):
    """ Downloads, unpacks and relocates
        a remote zip file """

    def install(self):
        super(ZIPInstaller, self).install()

        target_location = os.path.join(self.target_location, self.plugin_name)

        file = self.download_file(self.url)

        self.extract(file)

        self.move_tmp(self.tmp_dir, target_location)

        return True


class GITInstaller(BaseInstaller):
    """ Clones a git repository """

    def install(self):
        super(GITInstaller, self).install()

        print "Cloning %s from %s" % (self.plugin_name, self.url)

        self.run_command(
            ['git', 'clone', self.url, os.path.join(self.target_location, self.plugin_name)])


class SVNInstaller(BaseInstaller):
    """ Exports from the trunk of an svn repository """

    def install(self):
        super(SVNInstaller, self).install()

        print "Exporting %s from %s" % (self.plugin_name, self.svn_url)

        self._run_command(['svn', 'export', self.url, self.target_location])


class WPInstaller(BaseInstaller):
    """ Download and extract the specified plugin from the
        official WordPress SVN plugin repository """

    def install(self):
        super(WPInstaller, self).install()

        """ Build the url to the svn repo """
        svn_url = '/'.join([self.plugins_svn,
                            self.plugin_name.replace("\n", ''), 'trunk'])

        print "Exporting %s from Wordpress SVN" % (self.plugin_name)

        self.run_command(['svn', 'export', svn_url,
                          os.path.join(self.target_location, self.plugin_name)])

class FrameworkInstaller(ZIPInstaller):
    """ Downlads and installs the WP framework """

    def install(self):

        file = self.download_file(self.url)

        self.extract(file)

        self.move_tmp(
            os.path.join(self.tmp_dir, 'wordpress'), self.target_location)

        return True


class DBInstaller(BaseInstaller):
    """ Creates the wpm meta directory,
        and downlods the list of available plugins. """

    def install(self):

        data = self.download_file(self.plugins_svn)

        try:
            os.mkdir(self.wpm_meta_path)
        except OSError:
            print "WPM meta directory already exists (%s). Continuing"

        local = open(self.plugin_db_path, 'w+')

        list(local.read())

        plugins = re.findall('">(.*)/</a>', data)

        local.seek(0)
        local.write("\r\n".join(plugins))

        local.close()



#! /usr/bin/env python
import argparse
import os
import sys
import re
import shutil
import installers


class WPM(object):
    wp_latest = "http://wordpress.org/latest.zip"

    def __init__(self):

        parser = argparse.ArgumentParser(prog="wpm",
                                         description='A comand line tool to handle WordPress framework and plugin installation')

        subparsers = parser.add_subparsers(title="Available commands")

        install_framework_parser = subparsers.add_parser('installframework',
                                                         help="Install the latest version of WordPress")

        install_framework_parser.add_argument('location',
                                              help="Specify the target location.",
                                              default=os.getcwd())

        install_framework_parser.add_argument('-o', '--overwrite',
                                              action='store_true',
                                              help="Overwrite the previous installation of WordPress",
                                              default=False)

        install_framework_parser.set_defaults(func=self.install_framework)

        install_plugin_parser = subparsers.add_parser('installplugin',
                                                      help="Install a WordPress plugin")

        install_plugin_parser.add_argument('plugin',
                                           help="The plugin name",
                                           nargs='?',
                                           default=None)

        install_plugin_parser.add_argument('-l', '--location',
                                           help="Specify the target location.",
                                           default=os.getcwd())

        install_plugin_parser.add_argument('-o', '--overwrite',
                                           action='store_true',
                                           help="Overwrite the previous installation of WordPress",
                                           default=False)

        install_plugin_parser.add_argument('-r', '--requirements',
                                           help="Use a requirements file",
                                           default=False)

        install_plugin_parser.set_defaults(func=self.install_plugin)

        self.args = parser.parse_args()
        self.args.func(self.args)

    """
    Downloads and installs the WordPress framework.
    """

    def install_framework(self, parser, *args, **kwargs):

        installer = installers.FrameworkInstaller(url=self.wp_latest,
                                       target_location=self.args.location, overwrite=self.args.overwrite)

        if installer.install() is False:
            return False

        try:
            shutil.move(os.path.join(installer.target_location, 'wp-config-sample.php'),
                        os.path.join(installer.target_location, 'wp-config.php'))
            installer.set_security_keys(
                os.path.join(installer.target_location, 'wp-config.php'))
        except:
            print "Error while trying to add security keys to wp-config.php"

        print "Installed WordPress"

    """
    Installs a plugin
    """

    def install_plugin(self, parser, *args, **kwargs):

        if self.args.plugin is None and self.args.requirements is False:
            return

        if self.args.requirements:

            file = open(os.path.abspath(self.args.requirements), 'r')
            requirements = file.readlines()
            file.close()

            for line in requirements:
                self._route_install(line)
        else:
            self._route_install(self.args.plugin)

    """
    Search for available plugins to install
    """

    def search(self):

        local = open(os.path.expanduser('~/.wpm/available_plugins'), 'r+')
        search_str = sys.argv[2]

        for line in local.readlines():
            if re.search(search_str, line):
                print line.replace("\r\n", '')

    """
    List all installed plugins
    """

    def list(self):
        try:
            pass
        except:
            NotImplementedError

    """
    Update the local list of plugins
    """

    def update(self):

        print "Getting plugins list"
        installer = installers.DBInstaller(
            arguments=self.args, options=self.options)
        installer.install()
        print "\nPlugins updated."

    """
    Create the setup directory that contains the package repositories
    """

    def setup(self):

        print "Installing WPM"
        installer = installers.DBInstaller(
            arguments=self.args, options=self.options)
        installer.install()
        print "\nInstallation complete"

    """
    Decides which protocol to use to install the plugin
    """

    def _route_install(self, line):

        if line == '' or line[:1] == '#':
            return

        parsed = re.compile(
            '(?P<protocol>git|svn|zip)\+(?P<url>[^#]+)(?:#(?:name|egg)=(?P<name>.+))?').search(line.strip())

        params = {
            'arguments': self.args,
            'url': parsed.group('url') if parsed is not None else '',
            'plugin_name': parsed.group('name') if parsed is not None else '',
            'target_location': self.args.location,
            'overwrite': self.args.overwrite,
        }

        if parsed is None:
            params['plugin_name'] = line.strip()
            Installer = self._get_installer()
        else:
            Installer = self._get_installer(protocol=parsed.group('protocol'))

        instance = Installer(**params)
        instance.install()

        print "Installed " + params['plugin_name']

    def _get_installer(self, protocol='WP'):
        return getattr(installers, "%sInstaller" % protocol.upper())


if __name__ == '__main__':
    WPM()
